## data generating function
dgp <- function(n., t., w, beta=c(1,1,1), sar=0,
                sd.e=1, sd.mu=0, ar.e=0, lambda=0, add.e=50, unif.b=c(-0.5,0.5)) {

    ## check
    if(dim(w)[[1]]!=n.) stop("Dimensions of W inconsistent with those of data")

 ind <- rep(1:n., t.)
 tind <- rep(1, n.)
 for(i in 2:t.) tind <- c(tind, rep(i, n.))

 x1<-runif(t.*n., unif.b[1], unif.b[2])
 x2<-rnorm(t.*n.)

 dati <- data.frame(ind=ind, tind=tind, x1=x1, x2=x2)

 W <- kronecker(diag(1, t.), w)

 ## take add.e more at the beginning here, to make initial conditions irrelevant
 e <- rnorm(n.*(t.+add.e), sd=sd.e)

 ## make random effect

 mu.i <- rnorm(n., sd=sd.mu)
 mu <- rep(mu.i, t.)

 ## make autoregressive error

 ## current element is n.(j-1)+i; previous one is n.(j-2)+i

 for(i in 1:n.) {for(j in 2:(t.+add.e)) e[n.*(j-1)+i] = e[n.*(j-1)+i] + ar.e * e[n.*(j-2)+i]}

 ## discard first add.e time periods
 if(add.e>0) e <- e[-(1:(n.*add.e))]

 ## make spatial error
 ## u=lWu+e => u(I-lW)=e => u=(I-lW)^(-1)e

 u <- solve(diag(1,n.*t.)-lambda*W, e)

 ## make composite error

 err <- mu+u

 
 ## make y
 y <- rep(NA, n.*t.)
 nosar <- beta[1] + beta[2]*x1 + beta[3]*x2 + err
 for(i in 1:t.) {
     nosar.i <- nosar[1:n. + (i-1)*n.]
 y[1:n. + (i-1)*n.] <- solve(diag(1,n.) - sar*w, nosar.i)
 }

 dati$y <- y

 # dati$y <- solve(diag(1,n.*t.) - sar*W, beta[1] + beta[2]*x1 + beta[3]*x2 + err) # old inefficient version

 dati <- dati[order(dati$ind, dati$tind),]

 return(dati)
}

