## version of simdescribe.R meant to test the vcov estimate through size and power of the z-tests on spatial parms

## replicates table 9 in the JSS paper, if dir is set at /vc02/,
## table 10 if set at /vc06/

## set vcresults' dir
vcresdir <- "D:/test/vc02/"
#vcresdir <- "D:/test/vc_long06/"

sarparms <- c(-0.2, 0, 0.2) # ...or whatever
#sarparms <- c(-0.6, 0, 0.6) # ...or whatever
semparms <- sarparms

## init results' matrices
## ...for lambda
vcrestl <- matrix(ncol=length(sarparms), nrow=length(semparms))
## ...and rho
vcrestr <- matrix(ncol=length(sarparms), nrow=length(semparms))

## store number of negative variances
vcnegl <- matrix(ncol=length(sarparms), nrow=length(semparms))
vcnegr <- matrix(ncol=length(sarparms), nrow=length(semparms))

## store number of bad estimations (try-errors)
vctre <- matrix(ncol=length(sarparms), nrow=length(semparms))


## set convenience fun
is.tre <- function(x) class(x)=="try-error"
## errors pruning function
rm.tre <- function(x) {
    sumtr <- sum(unlist(lapply(x, is.tre)))
    y <- vector("list", length(x)-sumtr)
    kk <- 1
    for(k in 1:length(x)) {
        if(!is.tre(x[[k]])) {
            y[[kk]] <- x[[k]]
            kk <- kk+1
        }
    }
    return(y)
}

########################## RE(B) ML #################################

modelname <- "saremre"

## this snippet good for saremre, sarem2re

for(i in 1:length(sarparms)) {
  for(j in 1:length(semparms)) {

   load(paste(vcresdir, "vcres_", modelname, "_", as.character(sarparms[i]),
              "_", as.character(semparms[j]), ".rda", sep=""))

   ## count try-error cases
   tsumtr <- sum(unlist(lapply(vcres, is.tre)))
   vctre[j,i] <- tsumtr/length(vcres)
   ## prune errors
   vcres <- rm.tre(vcres)

    emp.lambda <- unlist(lapply(vcres, "[[", "arcoef"))
    emp.rho <- unlist(lapply(lapply(vcres, "[[", "errcomp"), "[[", "rho"))

    emp.vlambda <- unlist(lapply(vcres, "[[", "vcov.arcoef"))
    emp.vrho <- unlist(lapply(lapply(vcres, "[[", "vcov.errcomp"), "[", 4))

   vcnegl[j,i] <- sum(emp.vlambda<0)/length(emp.vlambda)
   vcnegr[j,i] <- sum(emp.vrho<0)/length(emp.vrho)

    emp.lambda <- emp.lambda[emp.vlambda>0]
    emp.vlambda <- emp.vlambda[emp.vlambda>0]
    emp.rho <- emp.rho[emp.vrho>0]
    emp.vrho <- emp.vrho[emp.vrho>0]

    emp.pl <- 2*pnorm(abs(emp.lambda)/sqrt(emp.vlambda), lower.tail=FALSE)
    emp.pr <- 2*pnorm(abs(emp.rho)/sqrt(emp.vrho), lower.tail=FALSE)
    vcrestl[j,i] <- sum(emp.pl<0.05)/length(emp.pl)  # lambda in columns
    vcrestr[j,i] <- sum(emp.pr<0.05)/length(emp.pr)  # rho in rows

  }
}

## prints results in Table 9, RE(B) ML (first row) and diagnostics

print(modelname)
print(vcrestl)
print(vcrestr)

print("Percentage of negative variances")
print(vcnegl)
print(vcnegr)

print("Percentage of try-errors")
print(vctre)

saremretab <- list(ml=vcrestl, mr=vcrestr)

########################## RE(KKP) ML ###################################

modelname <- "sarem2re"

## this snippet good for saremre, sarem2re

for(i in 1:length(sarparms)) {
  for(j in 1:length(semparms)) {

   load(paste(vcresdir, "vcres_", modelname, "_", as.character(sarparms[i]),
              "_", as.character(semparms[j]), ".rda", sep=""))

   ## count try-error cases
   tsumtr <- sum(unlist(lapply(vcres, is.tre)))
    vctre[j,i] <- tsumtr/length(vcres)
   ## prune errors
   vcres <- rm.tre(vcres)

    emp.lambda <- unlist(lapply(vcres, "[[", "arcoef"))
    emp.rho <- unlist(lapply(lapply(vcres, "[[", "errcomp"), "[[", "rho"))

    emp.vlambda <- unlist(lapply(vcres, "[[", "vcov.arcoef"))
    emp.vrho <- unlist(lapply(lapply(vcres, "[[", "vcov.errcomp"), "[", 4))

   vcnegl[j,i] <- sum(emp.vlambda<0)/length(emp.vlambda)
   vcnegr[j,i] <- sum(emp.vrho<0)/length(emp.vrho)

    emp.lambda <- emp.lambda[emp.vlambda>0]
    emp.vlambda <- emp.vlambda[emp.vlambda>0]
    emp.rho <- emp.rho[emp.vrho>0]
    emp.vrho <- emp.vrho[emp.vrho>0]

    emp.pl <- 2*pnorm(abs(emp.lambda)/sqrt(emp.vlambda), lower.tail=FALSE)
    emp.pr <- 2*pnorm(abs(emp.rho)/sqrt(emp.vrho), lower.tail=FALSE)
    vcrestl[j,i] <- sum(emp.pl<0.05)/length(emp.pl)  # lambda in columns
    vcrestr[j,i] <- sum(emp.pr<0.05)/length(emp.pr)  # rho in rows


  }
}

sarem2retab <- list(ml=vcrestl, mr=vcrestr)

## prints results in Table 9, RE(KKP) ML (2nd row) and diagnostics

print(modelname)
print(vcrestl)
print(vcrestr)

print("Percentage of negative variances")
print(vcnegl)
print(vcnegr)

print("Percentage of try-errors")
print(vctre)

############################## FE ML ###################################

modelname <- "saremfe"
## this one good for saremfe

for(i in 1:length(sarparms)) {
  for(j in 1:length(semparms)) {

    load(paste(vcresdir, "vcres_", modelname, "_", as.character(sarparms[i]), "_", as.character(semparms[j]), ".rda", sep=""))

   ## count try-error cases
   tsumtr <- sum(unlist(lapply(vcres, is.tre)))
   vctre[j,i] <- tsumtr/length(vcres)
   ## prune errors
   vcres <- rm.tre(vcres)

    emp.lambda <- unlist(lapply(lapply(vcres, "[[", "coefficients"), "[[", "lambda"))
    emp.rho <- unlist(lapply(lapply(vcres, "[[", "coefficients"), "[[", "rho"))

    emp.vlambda <- unlist(lapply(lapply(vcres, "[[", "vcov"), "[", 6))
    emp.vrho <- unlist(lapply(lapply(vcres, "[[", "vcov"), "[", 1))

   vcnegl[j,i] <- sum(emp.vlambda<0)/length(emp.vlambda)
   vcnegr[j,i] <- sum(emp.vrho<0)/length(emp.vrho)

    emp.lambda <- emp.lambda[emp.vlambda>0]
    emp.vlambda <- emp.vlambda[emp.vlambda>0]
    emp.rho <- emp.rho[emp.vrho>0]
    emp.vrho <- emp.vrho[emp.vrho>0]

    emp.pl <- 2*pnorm(abs(emp.lambda)/sqrt(emp.vlambda), lower.tail=FALSE)
    emp.pr <- 2*pnorm(abs(emp.rho)/sqrt(emp.vrho), lower.tail=FALSE)
    vcrestl[j,i] <- sum(emp.pl<0.05)/length(emp.pl)  # lambda in columns
    vcrestr[j,i] <- sum(emp.pr<0.05)/length(emp.pr)  # rho in rows

  }
}

saremfetab <- list(ml=vcrestl, mr=vcrestr)

## prints results in Table 9, FE ML (3rd row) and diagnostics

print(modelname)
print(vcrestl)
print(vcrestr)

print("Percentage of negative variances")
print(vcnegl)
print(vcnegr)

print("Percentage of try-errors")
print(vctre)

########################## RE GM ##################################

modelname <- "saremregm"
## this one good for saremre by gm

for(i in 1:length(sarparms)) {
  for(j in 1:length(semparms)) {

    load(paste(vcresdir, "vcres_", modelname, "_", as.character(sarparms[i]), "_", as.character(semparms[j]), ".rda", sep=""))

   ## count try-error cases
   tsumtr <- sum(unlist(lapply(vcres, is.tre)))
   vctre[j,i] <- tsumtr/length(vcres)
   ## prune errors
   vcres <- rm.tre(vcres)

   emp.lambda <- unlist(lapply(lapply(vcres, "[[", "coefficients"), "[[", "lambda"))
    #emp.rho <- unlist(lapply(lapply(vcres, "[[", "coefficients"), "[[", "rho"))

    emp.vlambda <- unlist(lapply(lapply(vcres, "[[", "vcov"), "[", 1))
    #emp.vrho ## empty

   vcnegl[j,i] <- sum(emp.vlambda<0)/length(emp.vlambda)

    emp.lambda <- emp.lambda[emp.vlambda>0]
    emp.vlambda <- emp.vlambda[emp.vlambda>0]

    emp.pl <- 2*pnorm(abs(emp.lambda)/sqrt(emp.vlambda), lower.tail=FALSE)
    #emp.pr <- 2*pnorm(abs(emp.rho)/sqrt(emp.vrho), lower.tail=FALSE)
    vcrestl[j,i] <- sum(emp.pl<0.05)/length(emp.pl)  # lambda in columns
    #vcrestr[j,i] <- sum(emp.pr<0.05)/length(emp.pr)  # rho in rows


  }
}

saremregmtab <- list(ml=vcrestl)

## prints results in Table 9, RE GM (4th row) and diagnostics

print(modelname)
print(vcrestl)

print("Percentage of negative variances")
print(vcnegl)

print("Percentage of try-errors")
print(vctre)

############################# FE GM #################################

modelname <- "saremfegm"
## this one good for saremfe by gm

for(i in 1:length(sarparms)) {
  for(j in 1:length(semparms)) {

    load(paste(vcresdir, "vcres_", modelname, "_", as.character(sarparms[i]), "_",
               as.character(semparms[j]), ".rda", sep=""))

   ## count try-error cases
   tsumtr <- sum(unlist(lapply(vcres, is.tre)))
   vctre[j,i] <- tsumtr/length(vcres)
   ## prune errors
   vcres <- rm.tre(vcres)

   emp.lambda <- unlist(lapply(lapply(vcres, "[[", "coefficients"), "[[", "lambda"))
    #emp.rho <- unlist(lapply(lapply(vcres, "[[", "coefficients"), "[[", "rho"))

    emp.vlambda <- unlist(lapply(lapply(vcres, "[[", "vcov"), "[", 1))
    #emp.vrho ## empty

   vcnegr[j,i] <- sum(emp.vlambda<0)/length(emp.vlambda)

    emp.lambda <- emp.lambda[emp.vlambda>0]
    emp.vlambda <- emp.vlambda[emp.vlambda>0]

    emp.pl <- 2*pnorm(abs(emp.lambda)/sqrt(emp.vlambda), lower.tail=FALSE)
    #emp.pr <- 2*pnorm(abs(emp.rho)/sqrt(emp.vrho), lower.tail=FALSE)
    vcrestl[j,i] <- sum(emp.pl<0.05)/length(emp.pl)  # lambda in columns
    #vcrestr[j,i] <- sum(emp.pr<0.05)/length(emp.pr)  # rho in rows

  }
}


saremfegmtab <- list(ml=vcrestl)

## prints results in Table 9, FE GM (5th row) and diagnostics

print(modelname)
print(vcrestl)

print("Percentage of negative variances")
print(vcnegl)

print("Percentage of try-errors")
print(vctre)

############### from here only pretty-printing of same results ################

## print xtables
library(xtable)

## re
pippo <- saremretab[[1]]
dimnames(pippo)[[1]]<-paste("rho=",semparms, sep="")
dimnames(pippo)[[2]]<-paste("lambda=",sarparms, sep="")

pluto <- saremretab[[2]]
dimnames(pluto)[[1]]<-paste("rho=",semparms, sep="")
dimnames(pluto)[[2]]<-paste("lambda=",sarparms, sep="")

xtable(pippo, digits=3)
xtable(pluto, digits=3)

## re2

pippo <- sarem2retab[[1]]
dimnames(pippo)[[1]]<-paste("rho=",semparms, sep="")
dimnames(pippo)[[2]]<-paste("lambda=",sarparms, sep="")

pluto <- sarem2retab[[2]]
dimnames(pluto)[[1]]<-paste("rho=",semparms, sep="")
dimnames(pluto)[[2]]<-paste("lambda=",sarparms, sep="")

xtable(pippo, digits=3)
xtable(pluto, digits=3)

## fe
pippo <- saremfetab[[1]]
dimnames(pippo)[[1]]<-paste("rho=",semparms, sep="")
dimnames(pippo)[[2]]<-paste("lambda=",sarparms, sep="")

pluto <- saremfetab[[2]]
dimnames(pluto)[[1]]<-paste("rho=",semparms, sep="")
dimnames(pluto)[[2]]<-paste("lambda=",sarparms, sep="")

xtable(pippo, digits=3)
xtable(pluto, digits=3)


## re GM
pippo <- saremregmtab[[1]]
dimnames(pippo)[[1]]<-paste("rho=",semparms, sep="")
dimnames(pippo)[[2]]<-paste("lambda=",sarparms, sep="")

xtable(pippo, digits=3)


## fe GM
pippo <- saremfegmtab[[1]]
dimnames(pippo)[[1]]<-paste("rho=",semparms, sep="")
dimnames(pippo)[[2]]<-paste("lambda=",sarparms, sep="")

xtable(pippo, digits=3)

